@@ -1,10 +1,10 @@
-use 5.20.0; # Dunno how to make tests pass for v5.22.1 to v5.24.0 so this is how it is
+use 5.20.0;
 use strict;
 use warnings;
 
 package Art::World {
 
-  our $VERSION = '0.18_14';
+  our $VERSION = '0.18_15';
 
   use Zydeco
     authority => 'cpan:SMONFF',
@@ -29,7 +29,7 @@ package Art::World {
   use Faker;
   use List::Util qw( max any );
   use Math::Round;
-  use Syntax::Keyword::Try qw( try );
+  use Try::Tiny;
   use Types::XSD qw( DateTime );
 
   role Abstraction {
@@ -58,23 +58,19 @@ package Art::World {
   }
 
   role Buyer {
-    requires money;           # Num $price
-    # TODO what if there are many buyers? Maybe we should remove the possibility
-    method acquire ( Artwork $art ) {
-      # TODO the money paid should go to previous owner
-      $art->change_owner( $self );
-      $self->money( $self->money - $art->value );
-      push $self->collection->@*, $art;
-      say "I bought for " . $art->value . " worth of art!" ;
+    requires money;
+
+    method $change_owner ( Agent *new_owner, Artwork *artwork ) {
+      $arg->artwork->owner->@* = ();
+      push $arg->artwork->owner->@*, $arg->new_owner ;
+      push $arg->new_owner->collection->@*, $arg->artwork;
+      # TODO guess it should bump some people reputation and artwork aura now
     }
 
-    # TODO maybe not so necessary, since this process is made through the change_owner() method
-    method sale ( Artwork $art ) {
-      $self->money( $self->money + $art->value );
-      # TODO DELETE FROM OWNER'S COLLECTION
-      # Using a grep filtering on the collection by id seems like a good idea
-      # ...
-      say "I sold !";
+    # TODO test the case when there are many buyers/owners
+    method acquire ( Artwork $art ) {
+      $self->pay( $art );
+      $art->$change_owner({ new_owner => $self, artwork => $art });
     }
   }
 
@@ -90,11 +86,6 @@ package Art::World {
       # Create a delegated method for each value in the enumeration
       handles         => 1,
       default         => 'for_sale' );
-    method change_owner ( Agent $new_owner ) {
-
-      $self->owner->@* = ();
-      push $self->owner->@*, $new_owner ;
-    }
   }
 
   role Crud {
@@ -102,7 +93,6 @@ package Art::World {
     has db! ( type => HashRef[ Str ], builder => true, lazy => true );
 
     method insert ( Str $table, HashRef $attributes ) {
-
       # TODO IT MUST BE TESTED
       unless ( $self->does( 'Art::World::Unserializable' )) {
         try {
@@ -170,8 +160,9 @@ package Art::World {
         }
         return $self->$attribute;
       } else {
+        require Art::World::Util;
         carp 'No such attribute ' . $attribute .
-          ' or we don\'t manage this kind of entity ' . Meta->get_class( $self );
+          ' or we don\'t manage this kind of entity ' . Art::World::Util->new_meta->get_class( $self );
       }
     }
 
@@ -188,6 +179,11 @@ package Art::World {
     }
   }
 
+  role Identity {
+    has id         ( type => Int );
+    has name!      ( type => Str );
+  }
+
   # From the documentation
   #
   # A `Place` must `invite()` all kind of `Agents` that produce `Work` or other kind
@@ -196,7 +192,6 @@ package Art::World {
   # `underground`, it will become an institution.
   #
   role Invitation {
-
     # In case a group of Agents are invited, for an Event like a performance, a concert
     multi method invite ( ArrayRef[ Agent ] *people,  Event *event  ) {
     }
@@ -221,8 +216,13 @@ package Art::World {
   }
 
   role Market {
-    #has money;
-    has price;
+    has money! ( type => Num, is => rw, default => 0 );
+    method pay ( Artwork $piece  ) {
+      # The money must be divided in equal between all owners
+      my $part = $piece->value / scalar $piece->owner->@*;
+      map { $_->money( $_->money - $part ) } $piece->owner->@*;
+      $self->money( $self->money - $piece->value );
+    }
   }
 
   role Showable {
@@ -273,9 +273,10 @@ package Art::World {
       }
     }
 
+    # TODO Guess this is more like an Agent method or role
     class Sex with Event;
 
-    class Playground with Crud {
+    class Playground with Crud, Identity {
 
       class Collective;
 
@@ -292,25 +293,20 @@ package Art::World {
         # Like, dunno, a city ? Could be in a Geo role I guess
         has location ( type => Str );
 
-        class Institution {
-
+        class Institution with Market {
           class Gallery with Exhibit, Buyer {
-
             has artwork ( type => ArrayRef );
             has artist ( type => ArrayRef );
             has event ( type => ArrayRef );
             has owner;
-            has money;
-
           }
 
-          class Museum with Exhibit;
+          class Museum with Exhibit, Buyer;
 
           class School {
             has student;
             has teachers;
           }
-
         }
 
         class Squat with Underground;
@@ -322,10 +318,8 @@ package Art::World {
 
     }
 
-    class Agent with Active, Crud, Fame {
-      # Should be required but will be moved to the Crudable area
-      has id         ( type => Int );
-      has name!      ( type => Str );
+    class Agent with Active, Crud, Fame, Identity {
+
       has reputation ( is => rw, type => PositiveOrZeroNum );
 
       # TODO Should be done during an event
@@ -356,7 +350,7 @@ package Art::World {
         }
       }
 
-      class Artist {
+      class Artist with Market {
 
         has artworks   ( type => ArrayRef );
         has collectors ( type => ArrayRef[ Collector ], default => sub { [] } );
@@ -389,8 +383,7 @@ package Art::World {
       }
 
 
-      class Collector with Active, Buyer {
-        has money! ( type => Num, is => rw );
+      class Collector with Active, Buyer, Market {
         has collection (
           type    => ArrayRef[ Artwork, 0 ],
           default => sub { [] },
@@ -454,39 +447,6 @@ package Art::World {
           type => ArrayRef[ Curator ] );
       }
     }
-
-    # Looks like the Art::World::Meta toolkit
-    # See also https://metacpan.org/pod/Class::MOP::Class for extreme cases
-    # my $meta = $self->meta;
-    class Meta {
-      # TODO See also Zydeco's $class object
-      method get_class( Object $klass ) {
-        return ref $klass;
-      }
-
-      method get_set_attributes_only( Object $clazz ) {
-        return keys %{ $clazz };
-      }
-
-      method get_all_attributes( Object $claxx ) {
-        return keys( %{
-          'Moo'->_constructor_maker_for(
-            $self->get_class( $claxx )
-              )->all_attribute_specs
-            });
-      }
-
-      method generate_discourse( ArrayRef $buzz = [] ) {
-        for ( 0 .. int rand( 3 )) { push $buzz->@*, Faker->new->company_buzzword_type1 };
-        return join ' ', $buzz->@*;
-      }
-
-      method random_set_of_agents ( Int $num ) { }
-
-      method titlify( Str $token ) {
-        return join '', map { ucfirst lc $_ } split /(\s+)/, $token;
-      }
-    }
   }
 }
 
@@ -505,8 +465,15 @@ Art::World - Modeling of creative processes
 
   my $artwork = Art::World->new_artwork(
     creator => [ $artist, $another_artist ]  ,
-    value => 100,
-    owner => 'smonff' );
+    value   => 100,
+    owner   => 'smonff' );
+
+  my $museum = Art::World->new_museum(
+    money => 1000,
+    name  => 'Contemporary Museum of Art::World'
+  );
+
+  $museum->acquire( $artwork );
 
 =head1 DESCRIPTION
 
@@ -523,11 +490,22 @@ L<HISTORY|Art::World::Manual/"HISTORY"> and the
 L<OBJECTIVES|Art::World::Manual/"OBJECTIVES"> section could be very handy to
 understand how this is an artwork using programming.
 
+=head1 MOTIVATIONS
+
+This project is a self-teaching experiment around the modern Perl
+object-oriented programming toolkit. In late 2020, I read a bit about
+L<Corrina|https://github.com/Ovid/Cor/wiki> and immediatly wanted to restart and
+experiment my old C<Art::World> project. While searching for something somewhat
+close to Corrina, since Corrina was only a draft and RFC, I discovered the
+Zydeco toolkit by Toby Inkster and immediatly felt very enthusiastic to use it
+to implement my idea. I hope it is a decent example of the possibilities this
+wonderful framework make possible.
+
 =head1 ROLES
 
 =head2 Abstraction
 
-This is were all kind of weird phenomenons happen. See the
+This is were all kind of weird phenomenons between abstract artistic entities happen. See the
 L<Manual|Art::World::Manual> about how it works.
 
 =head2 Active
@@ -540,22 +518,26 @@ public. Provide a C<participate> method.
 All this behavior and attributes are encapsulated in the C<Buyer> role because
 there is no such thing as somebody in the art world that buy but doesn't sale.
 
-The C<aquire> method is requiring some C<money>. C<acquire()> will remove the
-value of the C<Artwork> from the owner's money. The C<Artwork> is also added to
-it's C<Collection>.
+The C<aquire> method is requiring some C<money>. The C<money> is provided by the
+C<Market> role.
 
+  $collector->acquire( $artwork );
+
+When a C<Collector> C<acquire()> an C<Artwork>, the C<Artwork> C<ArrayRef> of
+owners is automatically cleared and substituted by the new owner(s).
+C<acquire()> will remove the value of the C<Artwork> from the owner's money.
+
+Those behaviors are delegated to the private C<$change_owner()> method, and to
+the C<pay()> method from the C<Market> role.
 
 
 =head2 Collectionable
 
+The C<collectionnable> provide a series of attributes.
+
 If it's collectionable, it can go to a C<Collector> collection or in a
 C<Museum>. A collectionable item is automatically owned by it's creator.
 
-When a C<Collector> C<acquire()> an C<Artwork>, the C<Artwork> C<ArrayRef> of
-owners is automatically cleared and substituted by the new owner(s).
-
-  $collector->acquire( $artwork );
-
 =head2 Crud
 
 The C<Crud> role makes possible to serialize most of the entities. For this to be
@@ -642,8 +624,9 @@ For C<Agents>, C<Places>, etc.
 
 =head2 Market
 
-It is all about offer and demand. Involve a price but should involve more money
-I guess.
+It is all about offer and demand. Involve C<money>. It provide a C<pay()> method
+that can be using by entities consumming this role. This method accept an
+C<Artwork> as an unique required parameter.
 
 =head2 Manager
 
@@ -845,7 +828,7 @@ Returns even non-set attributes for a particular object.
 
 Sébastien Feugère <sebastien@feugere.net>
 
-=head2 ACKNOWLEDGEMENTS
+=head1 ACKNOWLEDGEMENTS
 
 Thanks to everyone who has contributed to ack in any way, including Adrien
 Lucca, Toby Inkster, Ezgi Göç, Pierre Aubert, Seb. Hu-Rillettes, Joseph Balicki,
@@ -853,17 +836,6 @@ Nicolas Herubel and Nadia Boursin-Piraud.
 
 This project was made possible by the greatness of the L<Zydeco|https://zydeco.toby.ink/> toolkit.
 
-=head2 MOTIVATIONS
-
-This project is a self-teaching experiment around the modern Perl
-object-oriented programming toolkit. In late 2020, I read a bit about
-L<Corrina|https://github.com/Ovid/Cor/wiki> and immediatly wanted to restart and
-experiment my old C<Art::World> project. While searching for something somewhat
-close to Corrina, since Corrina was only a draft and RFC, I discovered the
-incredible Zydeco project by Toby Inkster and immediatly felt very enthusiastic
-to use it to implement my idea. I hope it is a decent example of the
-possibilities this wonderful framework make possible.
-
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2006-2021 Sebastien Feugère
